//---
//	hypervisor:wsiwtch - hypervisor world switch
//---
#include "fxBoot/hypervisor.h"

#include <gint/std/stdlib.h>
#include <gint/atomic.h>
#include <gint/drivers.h>

/* internal globals information */
HYPERVISOR_BSS void *hypervisor_current_vbr = NULL;
HYPERVISOR_BSS struct hworld *hypervisor_world_queue = NULL;

//---
//	World API
//---
/* hypervisor_wswitch_create(): Create a new world */
struct hworld *hypervisor_wswitch_create(void)
{
	struct hworld *world;

	world = hypervisor_world_queue;
	while (world != NULL) {
		if (world->private.status != HYPERVISOR_STATUS_DEAD) {
			world = world->private.next;
			continue;
		}
		world->private.status = HYPERVISOR_STATUS_INIT;
		return (world);
	}
	world = calloc(sizeof(struct hworld), 1);
	if (world != NULL) {
		world->private.status = HYPERVISOR_STATUS_CREATED;
		world->private.env = HYPERVISOR_ENV_CASIO;
	}
	return (world);
}

/* void hypervisor_wswitch_destroy(): Destroy the world */
int hypervisor_wswitch_destroy(struct hworld *world)
{
	if (world == NULL)
		return (-1);
	if (world->private.status != HYPERVISOR_STATUS_CREATED
			&&world->private.status != HYPERVISOR_STATUS_INIT) {
		return (-2);
	}
	if (world->memory.program.start != NULL)	// <-- move me
		free(world->memory.program.start);	// <-- loader unload
	if (world->context.drivers != NULL)
		drivers_context_destroy(world->context.drivers);
	world->private.inode = NULL;
	if (world->private.status == HYPERVISOR_STATUS_INIT) {
		world->private.status = HYPERVISOR_STATUS_DEAD;
		return (0);
	}
	free(world);
	return (0);
}

//---
//	Queue API
//---
/* hypervisor_wswitch_queue_register(): Register the new world */
int hypervisor_wswitch_queue_register(struct hworld *world)
{
	if (world == NULL)
		return (-1);
	world->private.next = hypervisor_world_queue;
	hypervisor_world_queue = world;
	return (0);
}

/* hypervisor_wswich_queue_get(): Get the world queue */
struct hworld *hypervisor_wswitch_queue_get(void)
{
	return (hypervisor_world_queue);
}

/* hypervisor_wswitch_queue_unregister(): Unregister a world */
int hypervisor_wswitch_queue_unregister(struct hworld *target)
{
	struct hworld **world;

	world = &hypervisor_world_queue;
	while (*world != NULL) {
		if (*world != target) {
			world = &(*world)->private.next;
			continue;
		}
		*world = (*world)->private.next;
		if (target->private.status != HYPERVISOR_STATUS_DEAD)
			hypervisor_wswitch_destroy(target);
		return (0);
	}
	return (-1);
}

//---
//	User API
//---
/* hypervisor_wswitch_check(): Check if a world switch is needed */
int hypervisor_wswitch_check(void)
{
	//TODO: scan the KEYSC registers
}

/* hypervisor_wswitch(): Perform a world switch */
int hypervisor_wswitch(void)
{
	//TODO: display menu
	//TODO: perform a world switch woth the next world context
}
